/* ESP8266_uMQTT_Broker_BME280
 *  
 * uMQTTBroker demo for Arduino (C++-style), BME280 szenzorral kiegészítve
 * 
 * A program egyedi MQTT brókert definiál, visszahívási függvényekkel.
 * A bróker funkciók ellátásán kívül feliratkozunk minden helyi témakörre 
 * és 5 másodpercenként publikáljuk a BME280 szernzorral mért adatokat
 * JSON formátumban: {"temperature":"xx.x","humidity":"xx","pressure":"xxxx.x"}
 * 
 * A programot az alábbi mintapélda kibővítésével alakítottuk ki: 
 * https://github.com/martin-ger/uMQTTBroker/tree/master/examples/uMQTTBrokerSampleOOFull
 * 
 * Tested with Arduino IDE 1.8.13 and ESP8266 Arduino core 2.7.4
 */

#include <ESP8266WiFi.h>
#include "uMQTTBroker.h"
#include "secrets.h"

#include <ArduinoJson.h>
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#define SEALEVELPRESSURE_HPA (1013.25)
Adafruit_BME280 bme;

StaticJsonDocument<200> doc;

//--- Your WiFi config here ------
char ssid[] = "ESP_uMQTT";      // network SSID in AP mode
char pass[] = "my_passwd";     // network password in AP mode
bool WiFiAP = false;           // Do yo want the ESP as AP?

/*
  Custom broker class with overwritten callback functions
*/
class myMQTTBroker: public uMQTTBroker {
  public:
    virtual bool onConnect(IPAddress addr, uint16_t client_count) {
      Serial.println(addr.toString() + " connected");
      return true;
    }

    virtual void onDisconnect(IPAddress addr, String client_id) {
      Serial.println(addr.toString() + " (" + client_id + ") disconnected");
    }

    virtual bool onAuth(String username, String password, String client_id) {
      Serial.println("Username / Password / ClientId: " + username + " / " + password + " / " + client_id);
      return (username.equals(String(uMQTT_USER)) && password.equals(String(uMQTT_PASS)));
    }

    virtual void onData(String topic, const char *data, uint32_t length) {
      char data_str[length + 1];
      os_memcpy(data_str, data, length);
      data_str[length] = '\0';
      Serial.println("received topic '" + topic + "' with data '" + (String)data_str + "'");
      //  printClients();
    }

    //--- Sample for the usage of the client info methods --------
    virtual void printClients() {
      for (int i = 0; i < getClientCount(); i++) {
        IPAddress addr;
        String client_id;
        getClientAddr(i, addr);
        getClientId(i, client_id);
        Serial.println("Client " + client_id + " on addr: " + addr.toString());
      }
    }
};

myMQTTBroker myBroker;

//---  WiFi init stuff --------------
void startWiFiClient() {
  Serial.println("Connecting to " + (String)ssid);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: " + WiFi.localIP().toString());
}

void startWiFiAP() {
  WiFi.mode(WIFI_AP);
  WiFi.softAP(ssid, pass);
  Serial.println("AP started");
  Serial.println("IP address: " + WiFi.softAPIP().toString());
}

void setup() {
  Serial.begin(115200);
  Serial.println();
  Serial.println();
  bme.begin(0x76);

  //--- Start WiFi -----------------
  if (WiFiAP)
    startWiFiAP();
  else
    startWiFiClient();
  //--- Start the broker ------------
  Serial.println("Starting MQTT broker");
  myBroker.init();
  //---  Subscribe to anything ---
  myBroker.subscribe("#");
}

int counter = 0;

void loop() {
  //---  Publish the counter value as String ------
  // myBroker.publish("broker / counter", (String)counter++);
  //--- wait a second
  delay(1000);
  //--- BME280 sensor data ---------
  doc["temperature"] = String(bme.readTemperature(),1);
  doc["humidity"] = (int)bme.readHumidity();
  doc["pressure"] = String(((bme.readPressure() + 1440) / 100.0F),1);
  String payload = doc.as<String>();
  myBroker.publish("cspista/home/bme280", payload);
  delay(4000);
}
